// This may look like C code, but it is really -*- C++ -*-
//
//	Verify construction of fractal maps and show them off
//
// $Id: vfractal_map.cc,v 2.2 1996/07/15 19:07:30 oleg Exp oleg $

#include "fractal_map.h"
#include "filter.h"
#include "iostream.h"
#include "MyImageViewer.h"

static void test_nonoise(const card order)
{
  cout << "\nTest constructing a map of order " << order << 
	  " without any noise\n" << endl;

  class ZeroNoise : public FractalMap
  {
  public:
    ZeroNoise(const card order) : FractalMap(order) {}
    ZeroNoise(const card order,const card smoothness,
  	      const card starting_scale_log)
      : FractalMap(order,smoothness,starting_scale_log) {}
    inline int get_noise(const card scale) const { return 0; }
  };

  IMAGE map = ZeroNoise(order);
  const GRAY pixel_mid_val = 1<<(map.q_depth()-1);
  verify_pixel_value(map,pixel_mid_val);
  IMAGE map1 = ZeroNoise(order,0,2);
  verify_pixel_value(map1,pixel_mid_val);
  cout << "\nDone" << endl;
}

static void test_norandom(void)
{
  const card order = 2;
  cout << "\nTest constructing a map of order " << order << 
	  " with non-random noise\n" << endl;

  class NoRNoise : public FractalMap
  {
  public:
    NoRNoise(const card order, const card smoothness,
  	     const card starting_scale_log,
	     const card bits_per_pixel)
      : FractalMap(order,smoothness,starting_scale_log,bits_per_pixel)
      { cout << "\tsmoothness " << smoothness << ", starting scale log"
             << starting_scale_log << ",depth " << bits_per_pixel << endl; }
    inline int get_noise(const card scale) const { return scale/2; }
  };

  {
    IMAGE map = NoRNoise(order,0,1,7);
    verify_pixel_value(map,96);
  }
  {
    IMAGE map = NoRNoise(order,0,2,7);
    verify_pixel_value(map,112);
  }
  {
    IMAGE map = NoRNoise(order,1,2,7);
    verify_pixel_value(map,96);
  }
  
#if 0
  map.print("map");

  IMAGE expected(map);
  expected = 2; expected(0,0) = 0;
  expected.square_of(3,rowcol(1,1)) = 4;
  assert( map == expected );
#endif

  cout << "\nDone" << endl;
}


				// Show off maps
static void demo_maps(void)
{
  cout << "Showing off different maps" << endl;

  class GaussNoise : public FractalMap
  {
  public:
    GaussNoise(const card order, const card smoothness,
  	     const card starting_scale_log)
      : FractalMap(order,smoothness,starting_scale_log) {}
				// Well-known Gaussian random number generator
				// Note rand() returns a number [0,2^15-1],
				// that is, within [0,1] with a 15-bit implicit
				// fraction
    inline int get_noise(const card scale) const {
      long sum = 0;
      for(register int i=0; i<12; i++)
	sum += rand();			// keep the result within 
      return (scale * (sum-(6<<15)))>>17; } // [-scale/2,scale/2]
  };

  WindowDestroyer window_destroyer;
  
  for(;;)
  {
    cout << "Enter a map's order (log2 of map's dimension) (0 to quit)" 
      << endl;
    int order;
    cin >> order;
    if( order <= 1 )
     return;
    cout << "Select random number generator: 0 - Uniform, 1 - Gaussian"
         << endl;
    int type;
    cin >> type;
    cout << "Enter smoothness and the starting scale log (both can be zero)"
         << endl;
    int smoothness, starting_scale_log;
    cin >> smoothness >> starting_scale_log;
    IMAGE map = type == 0 ? 
      (LazyImage&)FractalMap(order,smoothness,starting_scale_log) :
      (LazyImage&)FractalMap(order,smoothness,starting_scale_log);
    cout << "Select post-processing option" << endl;
    cout << "\t0 - no postprocessing" << endl;
    cout << "\t1 - apply blurring" << endl;
    cout << "\t2 - smooth non-deterministically" << endl;
    cin >> type;
    if( type == 1 )
      filter(map).conv(conv_kernel(1,1,1,CommonDenom(3))).
	clip_to_intensity_range();
    else if( type == 2 )
      filter(map).nondet_average(50);

    char buffer[70];
    sprintf(buffer,"smoothness %d, starting_scale_log %d",
    	    smoothness,starting_scale_log);
    window_destroyer.add(new MyImageWindow(map,buffer));
//    MyImageViewer image_viewer(map,buffer);
//    map.write_pgm("/tmp/abc",buffer);
//    system("/usr/local/bin/X11/xv /tmp/abc");
//    map.display(buffer);
  }
}

int main(void)
{
 test_nonoise(4); 
 test_nonoise(9); 
 test_norandom();

#ifdef __MWERKS__				// Macintosh
//  GetDateTime((unsigned long *)&qd.randSeed);
//  Random(); Random(); Random(); Random();	// Just randomizing
#else
  srand(time(0));
//  srand(0);
  rand(); rand(); rand(); rand();
#endif

class MyImageViewer : public BApplication
{
public:
	    MyImageViewer(void)
	    : BApplication('mimv')
	    { Run(); }
	    void ReadyToRun(void)
	    {
	      demo_maps();
	   	 PostMessage(B_QUIT_REQUESTED);
	    }
	    
};
MyImageViewer viewer;

printf("\nDone\n\n");
return 0;
 
}
