/*
 ***********************************************************************
 *
 *			Displaying a full-index-color image as a custom dialog item
 *
 ***********************************************************************
 */

#include "image.h"
#include "MyImageViewer.h"

/*--------------------------------------------------------*/

ScreenRect::ScreenRect(const IMAGE& image)
{
  left = top = 0;
  right = image.q_ncols();
  bottom = image.q_nrows();
}

					// Shifting a rectangle by the same amount
					// in X and Y
ScreenRect& ScreenRect::operator += (const int offset)
{ 
  top += offset; bottom += offset; left += offset; right += offset;
  return *this;
}


					// Print what rectangle is this
void ScreenRect::print(const char * title) const
{
  printf("Rectangle %s (%d,%d) - (%d,%d)",title,top,left,bottom,right);
}


								// Kind of a formal constructor, merely an initialization
MyImageView::MyImageView(const IMAGE& image)
   : BView((ScreenRect)image,0,B_FOLLOW_ALL,B_WILL_DRAW),
     BBitmap((ScreenRect)image,B_RGB_32_BIT)
{


    					// Move pixels from IMAGE to a off-screen pixmap
    					// Note that the rows of pixmap may be padded, by
    					// as much as off_to_next_row bytes. So we need
    					// to skip over the padding as we move pixels row
    					// by row
  Image_istream im_stream(image);
  const card height = BView::Bounds().IntegerHeight();
  const card width  = BView::Bounds().IntegerWidth();
  const card off_to_next_row = BytesPerRow()/sizeof(RGB32Pixel) - width;
  RGB32Pixel * pp = (RGB32Pixel *)Bits();
  for(register card row=0; row<height; row++, pp += off_to_next_row)
    for(register card col=0; col<width; col++)
     *pp++ = im_stream.get();
 
  assert( im_stream.eof() ); 
}

